<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Group;
use App\Models\GroupMember;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Session;

class GroupsController extends Controller
{
    public function store(Request $request)
    {
        try {
            // Validate the incoming request
            $validator = Validator::make($request->all(), [
                'title' => 'required|string|max:255',
                'user_id' => 'required|exists:users,id',
                // 'privacy' => 'required',
                // 'visibility' => 'required'
            ]);
            
            
        
            // If validation fails, return errors with 422 status
            if ($validator->fails()) {
                return response()->json([
                    'errors' => $validator->errors()
                ], 422);
            }
            
            $group = Group::create([
                'creator_id' => $request->user_id,
                'title' => $request->title,
                'description' => $request->description,
                'privacy' => 'public',
                'visibility' => 'visible',
            ]);
            
            if ($request->file('image')) {
                
                $path = $request->file('image')->store('group_images', 'public'); // Save to storage/app/public/post_images
                $imagePath = $path;
                $group->image = $imagePath;
            }
            
            $group->save();
            
            GroupMember::create([
                'group_id' => $group->id,
                'user_id' => $request->user_id,
            ]);
        
            return response()->json([
                'message' => 'Group created successfully',
                'group' => $group,
            ], 200);
        
        } catch (\Exception $e) {
        
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
    
    public function getAllGroups()
    {
        $groups = Group::where('privacy', 'public')->where('visibility', 'visible')->get();
        return response()->json($groups->load('members'));
    }
    
    public function getUserGroups($userId)
    {
        $user = User::where('id', $userId)->first();
        $userGroups = $user->groups;
        // dd($userGroups);
        return response()->json($userGroups->load('members'));
    }
    
    public function showGroup($groupId)
    {
        $group = Group::where('id', $groupId)->with('posts', 'members')->first();
        
        return response()->json($group);
    }
    
    public function addMember(Request $request, $groupId, $userId)
    {
        $group = Group::findOrFail($groupId);
        
        $groupMember = GroupMember::where('group_id', $groupId)->where('user_id', $userId)->first();
        
        if($groupMember){
            
            return response()->json(['message' => 'User is already in this group', 'member' => $groupMember]);
        }

        $groupMember = GroupMember::create([
            'group_id' => $groupId,
            'user_id' => $userId,
        ]);

        return response()->json(['message' => 'User added to the group successfully', 'member' => $groupMember]);
    }
    
    
}
