<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Chatify\Http\Controllers\MessagesController as ChatifyMessages;
use Chatify\Facades\ChatifyMessenger;
use Illuminate\Support\Facades\Auth;

class MessageController extends Controller
{
    /**
     * Send a message using Chatify's MessagesController.
     */
    public function sendMessage(Request $request)
    {
        $request->validate([
            'receiver_id' => 'required|integer',
            'message' => 'required|string',
        ]);

        // Use Chatify's MessagesController to send the message
        $messageController = new ChatifyMessages();
        $response = $messageController->send($request);

        $receiverDeviceToken = ''; // Fetch from DB

        if ($receiverDeviceToken) {
            $firebaseService = new \App\Services\FirebaseService();
            $firebaseService->sendNotification(
                $receiverDeviceToken,
                'New Message',
                $request->message
            );
        }

        return response()->json($response, 200);
    }

    /**
     * Get all messages for a conversation.
     */
    public function getMessages($conversationId)
    {
        // Fetch messages from ChatifyMessenger
        $messages = ChatifyMessenger::fetchMessagesQuery($conversationId)->get();

        return response()->json($messages, 200);
    }

    /**
     * Get all conversations for the authenticated user.
     */
    public function getConversations()
    {
        // Get the authenticated user's ID
        $userId = Auth::id();

        // Fetch conversations from ChatifyMessenger
        $conversations = ChatifyMessenger::conversations($userId);

        return response()->json($conversations, 200);
    }

    /**
     * Mark a message as read.
     */
    public function markAsRead($messageId)
    {
        // Mark the message as read using ChatifyMessenger
        ChatifyMessenger::markAsRead($messageId);

        return response()->json(['status' => 'Message marked as read'], 200);
    }
}
