<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Services\FirebaseService;
use App\Models\User;

class NotificationController extends Controller
{
    protected $firebaseService;

    public function __construct(FirebaseService $firebaseService)
    {
        $this->firebaseService = $firebaseService;
    }

    /**
     * Send a notification using FirebaseService.
     */
    public function sendNotification(Request $request)
    {
        $request->validate([
            'device_token' => 'required|string',
            'title' => 'required|string',
            'body' => 'required|string',
        ]);

        // Send the notification using the FirebaseService
        $this->firebaseService->sendNotification(
            $request->device_token,
            $request->title,
            $request->body
        );

        return response()->json(['message' => 'Notification sent successfully!'], 200);
    }
    
    public function getAllNotifications($userId)
    {
        $user = User::find($userId);
        $notifications = $user->notifications()->get();
        
        return response()->json($notifications);
    }
    
    public function updateNotification($userId, $id)
    {
        $user = User::find($userId);
        $notification = $user->notifications()->find($id);
        
        if ($notification) {
            $notification->markAsRead();
            
            return response()->json($notification);
        }
        
        return response()->json(['error' => 'Notification not found'], 404);
    }
    
    public function deleteNotification($userId, $id)
    {
        $user = User::find($userId);
        $notification = $user->notifications()->find($id);
        
        if ($notification) {
            $notification->delete();
            
            return response()->json(['success' => 'Notification deleted Successfully'], 200);
        }
        return response()->json(['error' => 'Notification not found'], 404);
    }
}