<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Post extends Model
{
    use HasFactory;
    
    protected $fillable = ['content', 'user_id', 'group_id', 'privacy', 'type'];
    
    public function user()
    {
        return $this->belongsTo(User::class);
    }

    // A post can have many images
    public function images()
    {
        return $this->hasMany(PostImage::class);
    }

    // A post can have many tags (people tagged in the post)
    public function tags()
    {
        return $this->hasMany(Tag::class);
    }

    // A post can have one location (check-in)
    public function location()
    {
        return $this->hasOne(Location::class);
    }

    // A post has one setting (privacy settings)
    public function postSetting()
    {
        return $this->hasOne(PostSetting::class);
    }
    
    public function comments()
    {
        return $this->hasMany(Comment::class);
    }
    
    public function likes()
    {
        return $this->hasMany(Like::class);
    }
    
    public function group()
    {
        return $this->belongsTo(Group::class);
    }
}
