<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'name',
        'email',
        'phone',
        'status',
        'device_token',
        'password',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * Get the attributes that should be cast.
     *
     * @return array<string, string>
     */
    protected function casts(): array
    {
        return [
            'email_verified_at' => 'datetime',
            'password' => 'hashed',
        ];
    }
    
    public function posts()
    {
        return $this->hasMany(Post::class);
    }

    // A user can be tagged in many posts
    public function tags()
    {
        return $this->hasMany(Tag::class);
    }
    
    public function groupsCreated()
    {
        return $this->hasMany(Group::class, 'creator_id');
    }
    
    public function groups()
    {
        return $this->belongsToMany(Group::class, 'group_members');
    }
    
    public function sharedPosts(){
        
        return $this->hasMany(Share::class);
    }
    
}
