<?php

namespace App\Notifications;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;
use Illuminate\Notifications\Messages\BroadcastMessage;

class UserTaggedInPost extends Notification implements ShouldQueue
{
    use Queueable;

    protected $post;
    protected $sender;

    public function __construct($post, $sender)
    {
        $this->post = $post;
        $this->sender = $sender;
    }

    public function via($notifiable)
    {
        return ['mail', 'database', 'broadcast'];  // Email, Database, and Real-time notification
    }

    public function toMail($notifiable)
    {
        return (new MailMessage)
            ->line("You were tagged in a post by {$this->sender->name}.")
            ->action('View Post', url("/posts/{$this->post->id}"))
            ->line('Thank you for using our application!');
    }

    public function toDatabase($notifiable)
    {
        return [
            'post_id' => $this->post->id,
            'sender_id' => $this->sender->id,
            'message' => "tagged you in a post.",
            'sender_image' => $this->sender->image ?? 'default-image-path.jpg',
            'sender_name' => $this->sender->name ?? 'Unknown User',
        ];
    }

    public function toBroadcast($notifiable)
    {
        return new BroadcastMessage([
            'post_id' => $this->post->id,
            'sender_id' => $this->sender->id,
            'message' => "tagged you in a post.",
            'sender_image' => $this->sender->image ?? 'default-image-path.jpg',
            'sender_name' => $this->sender->name ?? 'Unknown User',
        ]);
    }
}
