<?php

namespace App\Services;

use Kreait\Firebase\Messaging\CloudMessage;
use Kreait\Laravel\Firebase\Facades\Firebase;
use Illuminate\Support\Facades\Log;

class FirebaseService
{
    /**
     * Send a Firebase Cloud Messaging (FCM) notification to a device.
     *
     * @param string $deviceToken The FCM device token.
     * @param string $title The title of the notification.
     * @param string $body The body of the notification.
     * @return void
     */
    public function sendNotification($deviceToken, $title, $body)
    {
        $messaging = Firebase::messaging();
        $message = CloudMessage::withTarget('token', $deviceToken)
            ->withNotification([
                'title' => $title,
                'body' => $body,
            ]);

        try {
            $messaging->send($message);
        } catch (\Exception $e) {
            Log::error('Error sending Firebase notification: ' . $e->getMessage());
        }
    }

}
