<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use Laravel\Sanctum\Http\Middleware\EnsureFrontendRequestsAreStateful;
use App\Http\Controllers\AuthController;
use App\Http\Controllers\ProductsController;
use App\Http\Controllers\PostsController;
use App\Http\Middleware\CheckUserRole;
use App\Http\Controllers\NotificationController;
use App\Http\Controllers\API\MessageController;
use App\Http\Controllers\ChatController;
use App\Http\Controllers\CommentsController;
use App\Http\Controllers\LikeController;
use App\Http\Controllers\ShareController;
use App\Http\Controllers\GroupsController;


Route::get('/user', function (Request $request) {
    return $request->user();
})->middleware('auth:sanctum');


Route::post('/send-otp', [AuthController::class, 'sendOtp']);
Route::post('/verify-otp', [AuthController::class, 'verifyOtp']);
Route::post('/register', [AuthController::class, 'register']);
Route::post('/login', [AuthController::class, 'login'])->name('login');

// Route::get('/get-group-posts', [PostsController::class, 'getGroupPosts']);

Route::post('/send-notification', [NotificationController::class, 'sendNotification']);

Route::middleware('auth:sanctum')->group(function () {
    
    Route::post('/logout', [AuthController::class, 'logout']);
    Route::get('/user', function (Request $request) {
        return $request->user();
    });
    Route::get('/products', [PostsController::class, 'products']);
    Route::post('/update-profile/{id}', [AuthController::class, 'updateProfile']);
    
    // posts
    Route::get('/posts', [PostsController::class, 'index']);
    Route::get('/show-post/{postId}', [PostsController::class, 'show']);
    Route::post('/create-post', [PostsController::class, 'store']);
    Route::post('/update-post/{post}', [PostsController::class, 'update']);
    Route::delete('/delete-post/{userId}/{post}', [PostsController::class, 'destroy']);
    
    Route::get('/get-group-posts', [PostsController::class, 'getGroupPosts']);
    
    Route::post('/send-message', [ChatController::class, 'sendMessage']);
    Route::get('/messages/{conversationId}', [ChatController::class, 'getMessages']);
    Route::get('/get-user-conversations', [ChatController::class, 'getConversations']);
    Route::post('/mark-as-read/{conversationId}/{userId}', [ChatController::class, 'markAsRead']);
    // Route::get('/messages/{userId1}/{userId2}', [ChatController::class, 'getMessagesBetweenUsers']);
    Route::get('/get-all-users', [AuthController::class, 'getUsers']);
    
    // Fetch notifications (both read and unread)
    Route::get('/get-all-notifications/{userId}', [NotificationController::class, 'getAllNotifications']);
    
    // Mark a specific notification as read
    Route::put('/notifications/{userId}/{id}', [NotificationController::class, 'updateNotification']);
    
    Route::delete('/delete-notification/{userId}/{id}', [NotificationController::class, 'deleteNotification']);
    
    Route::post('/add-comment/{postId}', [CommentsController::class, 'store']);
    Route::post('/comment-reply/{commentId}', [CommentsController::class, 'storeReply']);
    Route::post('/like/{postId}', [LikeController::class, 'store']);
    Route::post('/share/{postId}', [ShareController::class, 'store']);
    
    Route::post('/create-group', [GroupsController::class, 'store']);
    Route::get('/get-all-groups', [GroupsController::class, 'getAllGroups']);
    Route::get('/show-group/{groupId}', [GroupsController::class, 'showGroup']);
    Route::post('/add-member/{groupId}/{userId}', [GroupsController::class, 'addMember']);
    Route::get('/user-groups/{userId}', [GroupsController::class, 'getUserGroups']);
    
    Route::post('/share-post/{post}/{userId}', [ShareController::class, 'store']);
    Route::get('/get-shared-posts/{userId}', [ShareController::class, 'getSharedPosts']);
});
